package cn.gygxzc.envir.config.shiro.filter

import com.gxzc.zen.api.sys.service.IPermissionService
import com.gxzc.zen.common.util.SpringContextHolder
import cn.gygxzc.envir.config.shiro.common.ZenPermission
import org.apache.shiro.web.filter.PathMatchingFilter
import org.springframework.http.HttpMethod
import org.springframework.http.HttpStatus
import java.util.regex.Pattern
import javax.servlet.ServletRequest
import javax.servlet.ServletResponse
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse

/**
 * Url拦截 集中式权限管理
 * 权限判定自行处理, 不交给Realm处理
 * @author NorthLan
 * @date 2018/4/25
 * @url https://noahlan.com
 */
class UrlPermissionsFilter : PathMatchingFilter() {

    /**
     * 此物只应天上有
     * filter比servlet更先初始化,反正就是不能自动注入
     */
    private var permissionService: IPermissionService? = null
        get() {
            if (field == null) {
                field = SpringContextHolder.getBean(IPermissionService::class.java)
            }
            return field
        }

    /**
     * 所有的权限从redis缓存中获取，不需要RPC或者其他方式获取
     */
    override fun onPreHandle(request: ServletRequest?, response: ServletResponse?, mappedValue: Any?): Boolean {
        request as HttpServletRequest
        if (request.method.equals(HttpMethod.OPTIONS.name, true)) {
            return true
        }

        // 获取系统权限列表(数据库中定义的)
        val sysPerms = permissionService!!.getAllSysPermission()

        // 根据请求url获取 系统 需要判定权限的url
        val permission = getPermissionIfs(sysPerms, request.requestURI, request.method)

        // 该url在系统表内未配置 默认认为无需权限认证 移交给下边流程处理
        if (permission.isEmpty()) {
            return true
        }

        // 若需要权限判定,则获取用户权限进行判定 正常流程走完
        if (!checkPermission(permission)) {
            forbidden(response)
            return false
        }
        return true
    }

    /**
     * 根据requestUri
     * 获取系统定义的权限列表
     */
    private fun getPermissionIfs(perms: MutableList<ZenPermission>, requestUri: String, method: String): List<ZenPermission> {
        return perms.filter {
            if (it.uri != null) {
                val regEx = "^${it.uri!!.replace("\\{\\*}".toRegex(), "[a-zA-Z\\\\d]+")}\$"
                //                (Pattern.compile(regEx).matcher(requestUri).find() || requestUri.startsWith(it.uri + "/")) && method.equals(it.method, false)
                Pattern.compile(regEx).matcher(requestUri).find() && method.equals(it.method, false)
            } else {
                false
            }
        }
    }

    /**
     * 检查用户权限
     */
    private fun checkPermission(perms: List<ZenPermission>): Boolean {
        val userPermission = permissionService!!.getSubjectPermission()
        if (userPermission.isEmpty()) {
            return false
        }
        return perms.any { sys ->
            userPermission.any { user ->
                user.code == sys.code
            }
        }
    }

    private fun forbidden(response: ServletResponse?) {
        response as HttpServletResponse
        response.apply {
            characterEncoding = "UTF-8"
            contentType = "application/json"
            sendError(HttpStatus.FORBIDDEN.value(), "FORBIDDEN") // 403
        }
    }
}